<?php
/**
 * Copyright notice
 *
 * (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 */

namespace SGalinski\SgJobs\Updates;

use TYPO3\CMS\Core\Database\Connection;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Install\Updates\DatabaseUpdatedPrerequisite;
use TYPO3\CMS\Install\Updates\UpgradeWizardInterface;

/**
 * Class DepartmentUpdateWizard
 *
 * @package SGalinski\SgJobs\Updates
 * @author Kevin Ditscheid <kevin.ditscheid@sgalinski.de>
 */
class DepartmentUpdateWizard implements UpgradeWizardInterface {
	/**
	 * The wizard identifier
	 */
	public const IDENTIFIER = 'tx_sgjobs_departmentupdatewizward';

	/**
	 * Check if the job table has a deleted area field
	 *
	 * @return bool
	 */
	protected function hasDeletedAreaField(): bool {
		return $this->hasField('zzz_deleted_area');
	}

	/**
	 * Check if the job table has an area field
	 *
	 * @return bool
	 */
	protected function hasAreaField(): bool {
		return $this->hasField('area');
	}

	/**
	 * Check if the job database table has a field with the given name in it
	 *
	 * @param string $fieldName
	 * @return bool
	 */
	protected function hasField(string $fieldName): bool {
		$connection = $this->getDatabaseConnection();
		$tableColumns = $connection->getSchemaManager()->listTableColumns('tx_sgjobs_domain_model_job');
		if (\array_key_exists($fieldName, $tableColumns)) {
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Get the database query builder
	 *
	 * @return Connection
	 */
	protected function getDatabaseConnection(): Connection {
		$connectionPool = GeneralUtility::makeInstance(ConnectionPool::class);
		return $connectionPool->getConnectionForTable('tx_sgjobs_domain_model_job');
	}

	/**
	 * @return string
	 */
	public function getIdentifier(): string {
		return self::IDENTIFIER;
	}

	/**
	 * @return string
	 */
	public function getTitle(): string {
		return 'Migrate old fulltext job areas to department records';
	}

	/**
	 * @return string
	 */
	public function getDescription(): string {
		return '';
	}

	/**
	 * @return bool
	 */
	public function executeUpdate(): bool {
		$connection = $this->getDatabaseConnection();
		$queryBuilder = $connection->createQueryBuilder();
		$field = $this->hasDeletedAreaField() ? 'zzz_deleted_area' : 'area';
		$queryBuilder->getRestrictions()->removeAll();
		/**
		 * @var array $jobs
		 */
		$jobs = $queryBuilder->select('uid', 'pid', 'sys_language_uid', $field)
			->from('tx_sgjobs_domain_model_job')
			->execute()
			->fetchAll();

		$newDepartments = [];
		if (\count($jobs) > 0) {
			foreach ($jobs as $index => $job) {
				if ($job[$field]) {
					$identifier = $job[$field] . '-' . $job['pid'] . '-' . $job['sys_language_uid'];
					$newDepartments[$identifier] = [
						'title' => $job[$field],
						'pid' => $job['pid'],
						'sys_language_uid' => $job['sys_language_uid']
					];
				} else {
					unset($jobs[$index]);
				}
			}

			if (\count($jobs) > 0) {
				\array_walk(
					$newDepartments,
					function ($department) use (&$dbQueries, $connection) {
						$queryBuilder = $connection->createQueryBuilder();
						$queryBuilder->insert('tx_sgjobs_domain_model_department')->values($department)->execute();
						$dbQueries[] = $queryBuilder->getSQL();
					}
				);

				$queryBuilder->resetQueryParts();
				$departments = $queryBuilder->select('uid', 'pid', 'sys_language_uid', 'title')
					->from('tx_sgjobs_domain_model_department')
					->execute()
					->fetchAll();

				$identifiedDepartments = [];
				foreach ($departments as $department) {
					$identifier = $department['title'] . '-' . $department['pid'] . '-' . $department['sys_language_uid'];
					$identifiedDepartments[$identifier] = $department;
				}

				$dbQueries[] = $queryBuilder->getSQL();

				foreach ($jobs as $index => $job) {
					$identifier = $job[$field] . '-' . $job['pid'] . '-' . $job['sys_language_uid'];
					if (isset($identifiedDepartments[$identifier])) {
						$jobs[$index]['department'] = $identifiedDepartments[$identifier]['uid'];
						unset($jobs[$index][$field]);
					} else {
						unset($jobs[$index]);
					}
				}

				if (\count($jobs) > 0) {
					\array_walk(
						$jobs,
						function ($job) use ($connection) {
							$queryBuilder = $connection->createQueryBuilder();
							$queryBuilder->update('tx_sgjobs_domain_model_job')
							->set('department', (int) $job['department'])
							->where(
								$queryBuilder->expr()->eq(
									'uid',
									$queryBuilder->createNamedParameter($job['uid'], \PDO::PARAM_INT)
								)
							)->execute();
						}
					);
				}
			}
		}

		return TRUE;
	}

	/**
	 * @return bool
	 */
	public function updateNecessary(): bool {
		if (
			!GeneralUtility::makeInstance(ConnectionPool::class)
				->getConnectionForTable('tx_sgjobs_domain_model_department')
				->getSchemaManager()
				->tablesExist(['tx_sgjobs_domain_model_department'])
		) {
			return FALSE;
		}

		if (
			$this->hasAreaField() ||
			$this->hasDeletedAreaField()
		) {
			return TRUE;
		}

		return FALSE;
	}

	/**
	 * @return array|string[]
	 */
	public function getPrerequisites(): array {
		return [
			DatabaseUpdatedPrerequisite::class
		];
	}
}
